"""

    This tests the implemented ACLs
    Also these tests depend on each other and have to be run alphabetically

"""


import unittest
from os import environ
from token_helper import *
# from envs_vars import *

baseurl = "%s/univention/" % environ['TESTURL']
#baseurl = "%s/" % environ['TESTURL']
twofactor_baseurl = baseurl + "command/twofactor-admin/"
#proxies = {"http": "http://localhost:8079", "https": "https://localhost:8079"}
#proxies = {"http": "http://192.168.178.25:8081", "https": "https://192.168.178.25:8081"}
proxies = {}


def search(uid, userlist):
    return [element for element in userlist if element['uid'] == uid]


class TestACLSs(unittest.TestCase):
    cookies = None
    headers = None
    logged_in_user = None
    privacyidea_auth_token = None

    @classmethod
    def login(self, user, password):
        self.logged_in_user = user
        res = requests.post(baseurl + "auth", json={"options": {"username": user, "password": password}},
                            cookies={"UMCLang": "en-US"}, proxies=proxies, verify=False)
        assert 200 == res.status_code
        assert res.cookies is not None
        self.cookies = res.cookies
        # pin language since some error messages are translated on the server
        requests.utils.add_dict_to_cookiejar(self.cookies, {"UMCLang": "en-US"})
        self.headers = {'X-Xsrf-Protection': res.cookies.get("UMCSessionId")}

    def logout(self):
        res = requests.get(baseurl + "/logout", cookies=self.cookies, headers=self.headers, proxies=proxies, verify=False)
        self.assertEqual(200, res.status_code)


    # get auth cookie
    @classmethod
    def setUpClass(self):
        self.privacyidea_auth_token = auth()
        delete_all_tokens_for_user(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token, environ['TESTUSER1'])
        delete_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token)
        create_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token)

    def assert_statuscode(self, school, user, statuscode):
        # Create a clean slate for the user to check
        print('#################', user)
        print('#################', statuscode)
        delete_all_tokens_for_user(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token, user)
        delete_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token)
        create_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token)

        if school:
            url = twofactor_baseurl + "set_current_school"
            res = requests.post(url, json={"options": {"school": school}}, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
            print('#######################' + url)
            print('#######################' + str(res))
            self.assertEqual(statuscode, res.status_code)

        # create qr token
        params = {'options': {'uid': ("%s" % user)}}
        res = requests.post(twofactor_baseurl + "rollover_qr_token", json=params, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        self.assertEqual(statuscode, res.status_code)

        # assign hardware token
        params = {'options': {'uid': ("%s" % user), 'serial': serial, 'current_otp_value': get_current_otp()}}
        res = requests.post(twofactor_baseurl + "assign_hardware_token", json=params, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        self.assertEqual(statuscode, res.status_code)

    def assert_can_not_access(self, school, user):
        return self.assert_statuscode(school, user, 403)

    def assert_can_not_access_400(self, school, user):
        return self.assert_statuscode(school, user, 400)

    def assert_can_access(self, school, user):
        return self.assert_statuscode(school, user, 200)

    def assert_can_not_list_users(self, school):
        # try to set school, outcome doesn't matter here, we're just interested in the user list
        if school:
            url = twofactor_baseurl + "set_current_school"
            res = requests.post(url, json={"options": {"school": school}}, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)

        # access user list
        url = twofactor_baseurl + "query"
        res = requests.post(url, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        self.assertEqual(403, res.status_code)

    def is_allmighty(self, username, password):
        self.logout()
        # This user is employee and school admin of DEMOSCHOOL
        self.login(username, password)

        # can access teachers of DEMOSCHOOL
        self.assert_can_access(None, environ['TESTUSER2'])

        # can access staff of DEMOSCHOOL
        self.assert_can_access(None, environ['TESTEMPLOYEE'])

        # can access teachers of ts
        self.assert_can_access(None, environ['TESTUSER1'])

        # can access staff of ts
        self.assert_can_access(None, environ['TESTEMPLOYEE2'])

        # can see all the users
        url = twofactor_baseurl + "query"
        res = requests.post(url, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        self.assertEqual(200, res.status_code)
        users = res.json()['result']
        self.assertIsNotNone(users)
        self.assertGreater(len(users), 9)

        # can query users by string
        res = requests.post(url, json={"options":{"pattern":environ['TESTUSER1']}}, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        users = res.json()['result']
        self.assertEqual(1, len(users))
        # gets users from different schools
        # TODO: Correct
        # self.assertTrue(any(environ['TESTEMPLOYEE'] in user['uid'] for user in users))
        # self.assertTrue(any(environ['TESTEMPLOYEE2'] in user['uid'] for user in users))
        # self.assertTrue(any(environ['TESTEMPLOYEE3'] in user['uid'] for user in users))

    def test_admin_is_almighty(self):
        self.is_allmighty(environ['TESTADMIN'], environ['TESTADMINPASS'])

    def test_domain_admin_is_allmighty(self):
        self.is_allmighty(environ['TESTDOMAINADMIN'], environ['TESTDOMAINADMINPASS'])

    # def test_school_admin(self):
    #     self.logout()
    #     # This user is employee and school admin of DEMOSCHOOL
    #     self.login(environ['TESTEMPLOYEE'], environ['TESTEMPLOYEEPASS'])

    #     # can access teachers of own school
    #     self.assert_can_access("DEMOSCHOOL", environ['TESTUSER2'])

    #     # can access staff of own school
    #     self.assert_can_access("DEMOSCHOOL", environ['TESTEMPLOYEE2'])

    #     # can access students of own school
    #     self.assert_can_access("DEMOSCHOOL", environ['TESTSTUDENT2'])

    #     # can't see teachers of other school
    #     url = twofactor_baseurl + "query"
    #     res = requests.post(url, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
    #     users = res.json()['result']
    #     self.assertEqual(200, res.status_code)
    #     self.assertNotIn(environ['TESTUSER1'], users)

    #     # can't access teachers of other school
    #     self.assert_statuscode("ts", environ['TESTUSER1'], 400)
    #     # can't access admin Account Tokens
    #     self.assert_statuscode(None, 'Administrator', 400)
    #     # can't access Domain Admin Account Tokens
    #     self.assert_statuscode(None, environ['TESTDOMAINADMIN'], 400)
    #     # can't access students of other school
    #     self.assert_statuscode("ts", environ['TESTSTUDENT1'], 400)
    #     # can't access staff of other school
    #     self.assert_statuscode("ts", environ['TESTEMPLOYEE3'], 400)


    # def test_teacher(self):
    #     self.logout()
    #     # TODO: Investigate
    #     """
    #         For some reason UCS returns "{"status": 400, "message": "You are not authorized for this action.""...
    #         I'm at a loss why unauthorized is 400 and not 403

    #     """
    #     self.login(environ['TESTUSER1'], environ['TESTUSER1PASS'])
    #     # can't access teachers of own school
    #     self.assert_can_not_access_400("ts", environ['TESTUSER1'])
    #     # can't access teachers of other school
    #     self.assert_can_not_access_400("DEMOSCHOOL", environ['TESTUSER2'])
    #     # can't access admin Account Tokens
    #     self.assert_can_not_access_400(None, 'Administrator')
    #     # can't access Domain Admin Account Tokens
    #     self.assert_can_not_access_400(None, environ['TESTDOMAINADMIN'])
    #     # can't access students of own school
    #     self.assert_can_not_access_400("ts", environ['TESTSTUDENT1'])
    #     # can't access staff of own school
    #     self.assert_can_not_access_400("ts", environ['TESTEMPLOYEE2'])
    #     # can't access staff of other school
    #     self.assert_can_not_access_400("DEMOSCHOOL", environ['TESTUSER2'])

    # def test_student(self):
    #     self.logout()
    #     # TODO: Investigate
    #     """
    #         For some reason UCS returns "{"status": 400, "message": "You are not authorized for this action.""...
    #         I'm at a loss why unauthorized is 400 and not 403

    #     """
    #     self.login(environ['TESTSTUDENT1'], environ['TESTSTUDENT1PASS'])
    #     # can't access teachers of own school
    #     self.assert_can_not_access_400("ts", environ['TESTUSER1'])
    #     # can't access teachers of other school
    #     self.assert_can_not_access_400("DEMOSCHOOL", environ['TESTUSER2'])
    #     # can't access admin Account Tokens
    #     self.assert_can_not_access_400(None, 'Administrator')
    #     # can't access Domain Admin Account Tokens
    #     self.assert_can_not_access_400(None, environ['TESTDOMAINADMIN'])
    #     # can't access students of own school
    #     self.assert_can_not_access_400("ts", environ['TESTSTUDENT1'])
    #     # can't access staff of own school
    #     self.assert_can_not_access_400("ts", environ['TESTEMPLOYEE2'])
    #     # can't access staff of other school
    #     self.assert_can_not_access_400("DEMOSCHOOL", environ['TESTEMPLOYEE'])

    # def test_staff(self):
    #     self.logout()
    #     self.login(environ['TESTEMPLOYEE3'], environ['TESTEMPLOYEE3PASS'])
    #     # TODO: Investigate
    #     """
    #         For some reason UCS returns "{"status": 400, "message": "You are not authorized for this action.""...
    #         I'm at a loss why unauthorized is 400 and not 403

    #     """
    #     # can't access teachers of own school
    #     self.assert_can_not_access_400("ts", environ['TESTUSER1'])
    #     # can't access teachers of other school
    #     self.assert_can_not_access_400("DEMOSCHOOL", environ['TESTUSER2'])
    #     # can't access admin Account Tokens
    #     self.assert_can_not_access_400(None, 'Administrator')
    #     # can't access Domain Admin Account Tokens
    #     self.assert_can_not_access_400(None, environ['TESTDOMAINADMIN'])
    #     # can't access students of own school
    #     self.assert_can_not_access_400("ts", environ['TESTSTUDENT1'])
    #     # can't access staff of own school
    #     self.assert_can_not_access_400("ts", environ['TESTEMPLOYEE2'])
    #     # can't access staff of other school
    #     self.assert_can_not_access_400("DEMOSCHOOL", environ['TESTEMPLOYEE'])




#TODO: This only fixes the existing testcase, by changing the requirements we have to add more test cases
# for all the "member of mfa-user and member of mfa-admin stuff".




        # set a school
        # url = twofactor_baseurl + "set_current_school"
        # res = requests.post(url, json={"options": {"school": "ts"}}, cookies=self.cookies, headers=self.headers)
        # self.assertEqual(200, res.status_code)


if __name__ == '__main__':
    unittest.main()
