from os import environ
import unittest

import requests
from selenium import webdriver
from splinter import Browser
# from envs_vars import *

environ['PATH'] = environ['PATH'] + ":/opt/apps/selenium-driver"


class TestGui(unittest.TestCase):
    def login(self, username, password):
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument('ignore-certificate-errors')
        browser = Browser('chrome', chrome_options=chrome_options)
        browser.driver.implicitly_wait(10)

        browser.visit('%s/univention/login' % environ['TESTURL'])

        browser.find_by_id('umcLoginUsername').fill(username)
        browser.find_by_id('umcLoginPassword').fill(password)
        browser.find_by_name('submit').click()

        self.assertTrue(browser.is_element_present_by_css('.umcGalleryItem'), "Login failed")
        return browser

    def set_autosearch(self, browser, value):
        cookies = browser.cookies.all()
        res = requests.p("%s/univention/command/ucr/put" + environ['TESTURL'], cookies=cookies, headers={'X-Xsrf-Protection': cookies["UMCSessionId"]},
                            json={"options": [
                                {"object": {"key": "twofactor-admin/autosearch", "value": value, "description[en]": ""},
                                 "options": {"key": "twofactor-admin/autosearch"}}]})
        self.assertEqual(200, res.status_code)

    def test_autosearch(self):
        browser = self.login(environ['TESTADMIN'], environ['TESTADMINPASS'])
        self.set_autosearch(browser, "on")
        browser.find_by_css(".umcCategory-ucs-school-admin").click()
        browser.find_by_text("Two-factor administration").click()

        browser.find_by_css('[name=pattern]').fill(environ['TESTUSER1'])
        browser.find_by_css('.umcSearchBox:not(.dijitDisabled) span.ucsButton').click()
        self.assertTrue(browser.is_element_present_by_text(environ['TESTUSER1']))
        browser.find_by_css('[name=pattern]').fill(environ['TESTUSER2'])
        browser.find_by_css('.umcSearchBox:not(.dijitDisabled) span.ucsButton').click()
        self.assertTrue(browser.is_element_present_by_text(environ['TESTUSER2']))
        self.set_autosearch(browser, "off")
        browser.reload()
        self.assertTrue(browser.is_element_present_by_text("No entries could be found"))
        self.set_autosearch(browser, "on")
        browser.reload()


if __name__ == '__main__':
    unittest.main()
