import os
import time
import unittest
# from envs_vars import *
from selenium import webdriver
from splinter import Browser

import token_helper

# For creating the necessary Test Fixtures, see the fixtures Folder. All Userdata is supplied via Environment Variables

os.environ['PATH'] = os.environ['PATH'] + ":/opt/apps/selenium-driver"


def find_button_by_text(browser, text):
    return browser.find_by_xpath('//*[contains(@class, "dijitButtonText") and text() = \'%s\']' % text)


def find_exception_button_by_text(browser, text):
    return browser.find_by_xpath('//*[contains(@class, "dijitButtonNode")]contains(@class, "dijitReset")]')


class TestGui(unittest.TestCase):
    def setUp(self):
        super(unittest.TestCase, self)
        self.auth_token = token_helper.auth()
        token_helper.delete_all_tokens_for_user(os.environ['TESTPRIVACYIDEA_URL'], self.auth_token, os.environ['TESTUSER1'])
        token_helper.create_test_token(os.environ['TESTPRIVACYIDEA_URL'], self.auth_token)

    def selectSchool(self, browser, school):
        browser.find_by_css(".umcCategory-ucs-school-admin").click()
        browser.find_by_text("Two-factor administration").click()
        # we have to wait for the load before we can execute javascript successfully
        self.assertTrue(browser.is_element_present_by_css('[data-qa=schoolsubmit]', wait_time=45), "Module load failed")
        browser.execute_script("document.getElementsByName('schools')[0].value='%s'" % school)
        browser.find_by_css('[data-qa=schoolsubmit]').click()

    def login(self, username, password, otp=None):

        # profile = webdriver.FirefoxProfile()
        # profile.accept_untrusted_certs = True
        # driver = webdriver.Firefox(firefox_profile=profile)
        # browser = Browser('firefox', firefox_driver=driver)

        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument('ignore-certificate-errors')
        browser = Browser('chrome', chrome_options=chrome_options)
        browser.driver.implicitly_wait(10)

        if otp:
            browser.visit('%s/univention/saml' % os.environ['TESTURL'])
        else:
            browser.visit('%s/univention/login' % os.environ['TESTURL'])

        browser.find_by_id('umcLoginUsername').fill(username)
        browser.find_by_id('umcLoginPassword').fill(password)
        browser.find_by_name('submit').click()

        if otp is not None:
            self.assertTrue(browser.is_element_present_by_name('password'), 'No OTP Prompt')
            browser.find_by_name('password').fill(otp)
            browser.find_by_id('regularsubmit').click()

        self.assertTrue(browser.is_element_present_by_css('.umcGalleryItem'), "Login failed")
        return browser

    def testTokenActions(self):
        browser = self.login(os.environ['TESTEMPLOYEE2'], os.environ['TESTEMPLOYEE2PASS'])

        self.selectSchool(browser, "ts")

        browser.find_by_css('[name=pattern]').fill(os.environ['TESTUSER1'])
        browser.find_by_css('.umcSearchBox:not(.dijitDisabled) span.ucsButton').click()

        browser.find_by_text(os.environ['TESTUSER1']).click()

        # Generate QR-Code
        # Theres seems to be no way to attach data attributes to umc grid...
        find_button_by_text(browser, "QR-Code (\"Handy\") Token").click()

        self.assertTrue(browser.is_element_present_by_css('.dijitDialogTitleBar'), "Confirmation")
        find_button_by_text(browser, "OK").click()

        self.assertTrue(browser.is_element_present_by_css('[data-qa=qrcode-image]'), "No QR-Code Image")

        browser.find_by_xpath('//*[contains(@class, "dijitButtonContents")][@data-qa = "closebutton"]').click()
        time.sleep(1)

        # Assign Hardware Token
        browser.find_by_xpath('//*[contains(@class, "dgrid-cell")][text() = "%s"]' % os.environ['TESTUSER1']).click()
        find_button_by_text(browser, 'Hardware Token').click()

        find_button_by_text(browser, "Submit")

        browser.find_by_css('[data-qa=serialnumber]').type(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').type(token_helper.get_current_otp())

        find_button_by_text(browser, 'Submit').click()
        browser.find_by_xpath('//*[contains(@class, "dijitButtonText")][text() = "OK"]')

        # assert Token was assigned successfully
        self.assertTrue(browser.is_element_present_by_text('Token was assigned successfully'), "Token assigning failed")
        find_button_by_text(browser, "OK").click()

        # lost Token
        find_button_by_text(browser, "Lost Token").click()

        self.assertTrue(browser.is_element_present_by_css('.dijitDialogTitleBar'), "Confirmation")
        time.sleep(1)
        find_button_by_text(browser, "OK").click()

        self.assertTrue(browser.is_element_present_by_text('Token was reported lost/broken successfully'), "Lost Token Action failed")
        find_button_by_text(browser, "OK").click()

        # return lost token
        find_button_by_text(browser, "Return Hardware Token").click()

        browser.find_by_css('[data-qa=serialnumber]').fill(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').fill(token_helper.get_current_otp())

        find_button_by_text(browser, "Submit").click()

        self.assertTrue(browser.is_element_present_by_text('Token was returned successfully'), "Token return failed")
        find_button_by_text(browser, "OK").click()

        # Assign Hardware Token again
        browser.find_by_xpath('//*[contains(@class, "dgrid-cell")][text() = "%s"]' % os.environ['TESTUSER1']).click()
        find_button_by_text(browser, 'Hardware Token').click()

        find_button_by_text(browser, "Submit")

        browser.find_by_css('[data-qa=serialnumber]').type(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').type(token_helper.get_current_otp())

        find_button_by_text(browser, 'Submit').click()
        browser.find_by_xpath('//*[contains(@class, "dijitButtonText")][text() = "OK"]')

        # assert Token was assigned successfully
        self.assertTrue(browser.is_element_present_by_text('Token was assigned successfully'), "Token assigning failed")
        find_button_by_text(browser, "OK").click()

        # Return Token (which was not lost)
        find_button_by_text(browser, "Return Hardware Token").click()

        browser.find_by_css('[data-qa=serialnumber]').fill(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').fill(token_helper.get_current_otp())

        find_button_by_text(browser, "Submit").click()

        self.assertTrue(browser.is_element_present_by_text('Token was returned successfully'), "Token return failed")

        find_button_by_text(browser, "OK").click()

        browser.quit()

    def testReturnTokenErrorMessages(self):
        browser = self.login(os.environ['TESTEMPLOYEE2'], os.environ['TESTEMPLOYEE2PASS'])

        self.selectSchool(browser, "ts")

        browser.find_by_css('[name=pattern]').fill(os.environ['TESTUSER1'])
        browser.find_by_css('.umcSearchBox:not(.dijitDisabled) span.ucsButton').click()

        browser.find_by_text(os.environ['TESTUSER1']).click()

        # Return token
        find_button_by_text(browser, "Return Hardware Token").click()
        time.sleep(1)

        # enter nothing
        find_button_by_text(browser, "Submit").click()
        self.assertTrue(browser.is_element_present_by_css(".dijitValidationTextBoxError"))

        # enter Serial but no otp
        browser.find_by_css('[data-qa=serialnumber]').fill(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').click()
        find_button_by_text(browser, "Submit").click()
        self.assertTrue(browser.is_element_present_by_css(".dijitValidationTextBoxError"))

        # enter wrong serial with short otp
        browser.find_by_css('[data-qa=current_otp]').fill("1234")
        find_button_by_text(browser, "Submit").click()
        self.assertTrue(browser.is_element_present_by_css(".dijitValidationTextBoxError"))

        # enter wrong serial with wrong otp
        browser.find_by_css('[data-qa=serialnumber]').fill("6345634547")
        browser.find_by_css('[data-qa=current_otp]').fill("123456")
        find_button_by_text(browser, "Submit").click()

        self.assertTrue(browser.is_element_present_by_css('.umcServerErrorMessage'), 'Token not found.')
        find_button_by_text(browser, "Ok").click()

        # enter correct serial with wrong otp
        browser.find_by_css('[data-qa=serialnumber]').fill(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').fill("123456")
        find_button_by_text(browser, "Submit").click()

        self.assertTrue(browser.is_element_present_by_text('OTP Value invalid'))
        find_button_by_text(browser, "Ok").click()

        browser.quit()

    # this just for manual debugging and as a reminder that not all error scenarios are tested by automated tests
    def testLostTokenErrorMessages(self):
        browser = self.login(os.environ['TESTEMPLOYEE2'], os.environ['TESTEMPLOYEE2PASS'])

        self.selectSchool(browser, "ts")

        browser.find_by_css('[name=pattern]').fill(os.environ['TESTUSER1'])
        browser.find_by_css('.umcSearchBox:not(.dijitDisabled) span.ucsButton').click()

        browser.find_by_text(os.environ['TESTUSER1']).click()
        #token_helper.unassign_test_token(os.environ['TESTPRIVACYIDEA_URL'], self.auth_token, os.environ['TESTUSER1'])
        find_button_by_text(browser, "Lost Token").click()
        time.sleep(1)

        find_button_by_text(browser, "OK").click()

        # user has no active tokens -> this does not really need an error message
        self.assertTrue(browser.is_element_present_by_css('.umcServerErrorMessage'), 'There are no Tokens assigned to the User.')
        find_button_by_text(browser, "Ok").click()
        browser.quit()

        # disable token failed -> a race condition which is very tricky to reproduce (somebody has to delete the token after the disable loop was entered)
        # practically only testable with a debugger

        # delete token failed -> a race condition which is very tricky to reproduce (somebody has to delete the token after the disable loop was entered)


    def testAssignHardwareTokenErrorMessages(self):
        browser = self.login(os.environ['TESTEMPLOYEE2'], os.environ['TESTEMPLOYEE2PASS'])
        self.selectSchool(browser, "ts")

        # display all users
        # browser.find_by_id('umc_widgets_SearchBox_1').fill(os.environ['TESTUSER1'])
        # browser.find_by_id('umc_widgets_Button_17').click()
        browser.find_by_css('[name=pattern]').fill(os.environ['TESTUSER1'])
        browser.find_by_css('.umcSearchBox:not(.dijitDisabled) span.ucsButton').click()
        # Assign Hardware Token
        browser.find_by_xpath('//*[contains(@class, "dgrid-cell")][text() = "%s"]' % os.environ['TESTUSER1']).click()
        find_button_by_text(browser, 'Hardware Token').click()
        time.sleep(1)
        # enter nothing
        find_button_by_text(browser, "Submit").click()
        self.assertTrue(browser.is_element_present_by_css(".dijitValidationTextBoxError"))

        # enter Serial but no otp
        browser.find_by_css('[data-qa=serialnumber]').fill(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').click()
        find_button_by_text(browser, "Submit").click()
        self.assertTrue(browser.is_element_present_by_css(".dijitValidationTextBoxError"))

        # enter wrong serial with short otp
        browser.find_by_css('[data-qa=current_otp]').fill("1234")
        find_button_by_text(browser, "Submit").click()
        self.assertTrue(browser.is_element_present_by_css(".dijitValidationTextBoxError"))

        # enter wrong serial with wrong otp
        browser.find_by_css('[data-qa=serialnumber]').fill("6345634547")
        browser.find_by_css('[data-qa=current_otp]').fill("123456")
        find_button_by_text(browser, "Submit").click()

        #self.assertTrue(browser.is_element_present_by_css('.dijitDialogTitleBar'), "An error occurred")
        self.assertTrue(browser.is_element_present_by_css('.umcServerErrorMessage'), "Token not found.")
        find_button_by_text(browser, "Ok").click()

        # enter correct serial with wrong otp
        browser.find_by_css('[data-qa=serialnumber]').fill(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').fill("123456")
        find_button_by_text(browser, "Submit").click()

        #self.assertTrue(browser.is_element_present_by_text('OTP Value invalid'))
        self.assertTrue(browser.is_element_present_by_css('.umcServerErrorMessage'), "OTP Value invalid")
        find_button_by_text(browser, "Ok").click()

        # try to assign the same token twice
        browser.find_by_css('[data-qa=serialnumber]').fill(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').fill(token_helper.get_current_otp())
        find_button_by_text(browser, "Submit").click()

        self.assertTrue(browser.is_element_present_by_text('Token was assigned successfully'))
        find_button_by_text(browser, "OK").click()
        find_button_by_text(browser, 'Hardware Token').click()
        time.sleep(1)
        browser.find_by_css('[data-qa=serialnumber]').fill(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').fill(token_helper.get_current_otp())
        find_button_by_text(browser, "Submit").click()
        self.assertTrue(browser.is_element_present_by_css('.umcServerErrorMessage'), 'This User is already assigned to this Token')
        find_button_by_text(browser, "Ok").click()
        find_button_by_text(browser, "Cancel").click()

        # try assign already assigned token to another user
        browser.find_by_css('[name=pattern]').fill(os.environ['TESTUSER3'])
        browser.find_by_css('.umcSearchBox:not(.dijitDisabled) span.ucsButton').click()
        #browser.find_by_xpath('//*[contains(@class, "dgrid-cell")][text() = "%s"]' % os.environ['TESTUSER3']).click()
        browser.find_by_text(os.environ['TESTUSER3']).click()
        find_button_by_text(browser, 'Hardware Token').click()
        time.sleep(1)

        # enter Serial but no otp
        browser.find_by_css('[data-qa=serialnumber]').fill(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').fill(token_helper.get_current_otp())
        find_button_by_text(browser, "Submit").click()
        self.assertTrue(browser.is_element_present_by_text('Token is already assigned to another user. Please return it before assigning it again.'))

        browser.quit()

    def testQrCodeTokenErrors(self):

        browser = self.login(os.environ['TESTEMPLOYEE2'], os.environ['TESTEMPLOYEE2PASS'])
        self.selectSchool(browser, "ts")

        browser.find_by_css('[name=pattern]').fill(os.environ['TESTUSER1'])
        browser.find_by_css('.umcSearchBox:not(.dijitDisabled) span.ucsButton').click()

        browser.find_by_text(os.environ['TESTUSER1']).click()
        token_helper.assign_test_token(os.environ['TESTPRIVACYIDEA_URL'], self.auth_token, os.environ['TESTUSER1'])

        find_button_by_text(browser, "QR-Code (\"Handy\") Token").click()

        self.assertTrue(browser.is_element_present_by_css('.dijitDialogTitleBar'), "Confirmation")
        find_button_by_text(browser, "OK").click()

        #self.assertTrue(browser.is_element_present_by_css('.umcServerErrorMessage'), "Can't create QR Token since the User has an active Hardware Token")
        self.assertTrue(browser.is_element_present_by_css('.umcServerErrorMessage'), "Can't create QR Token since the User has an active Hardware Token.")

        browser.quit()

    def adminActions(self, username, password):

        # admin sees all users without choosing a school
        browser = self.login(username, password)

        browser.find_by_css(".umcCategory-ucs-school-admin").click()
        browser.find_by_text("Two-factor administration").click()
        # admin can roll out qr-code token
        #browser.find_by_text(os.environ['TESTUSER1']).click()
        #browser.find_by_id('umc_widgets_SearchBox_2').fill(os.environ['TESTUSER1'])
        browser.find_by_css('[name=pattern]').fill(os.environ['TESTUSER1'])
        browser.find_by_css('.umcSearchBox:not(.dijitDisabled) span.ucsButton').click()
        browser.find_by_text(os.environ['TESTUSER1']).click()

        # Generate QR-Code
        # Theres seems to be no way to attach data attributes to umc grid...
        find_button_by_text(browser, "QR-Code (\"Handy\") Token").click()

        self.assertTrue(browser.is_element_present_by_css('.dijitDialogTitleBar'), "Confirmation")
        find_button_by_text(browser, "OK").click()

        self.assertTrue(browser.is_element_present_by_css('[data-qa=qrcode-image]'), "Please ask the user to scan the QR-Code.")

        browser.find_by_xpath('//*[contains(@class, "dijitButtonContents")][@data-qa = "closebutton"]').click()
        time.sleep(1)

        # Assign Hardware Token
        browser.find_by_xpath('//*[contains(@class, "dgrid-cell")][text() = "%s"]' % os.environ['TESTUSER1']).click()
        find_button_by_text(browser, 'Hardware Token').click()

        find_button_by_text(browser, "Submit")

        browser.find_by_css('[data-qa=serialnumber]').type(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').type(token_helper.get_current_otp())

        find_button_by_text(browser, 'Submit').click()
        browser.find_by_xpath('//*[contains(@class, "dijitButtonText")][text() = "OK"]')

        # assert Token was assigned successfully
        self.assertTrue(browser.is_element_present_by_text('Token was assigned successfully'), "Token assigning failed")
        find_button_by_text(browser, "OK").click()

        # lost Token
        find_button_by_text(browser, "Lost Token").click()

        self.assertTrue(browser.is_element_present_by_css('.dijitDialogTitleBar'), "Confirmation")
        time.sleep(1)
        find_button_by_text(browser, "OK").click()

        self.assertTrue(browser.is_element_present_by_text('Token was reported lost/broken successfully'), "Lost Token Action failed")
        time.sleep(1)
        find_button_by_text(browser, "OK").click()

        # return lost token
        find_button_by_text(browser, "Return Hardware Token").click()

        browser.find_by_css('[data-qa=serialnumber]').fill(token_helper.serial)
        browser.find_by_css('[data-qa=current_otp]').fill(token_helper.get_current_otp())

        find_button_by_text(browser, "Submit").click()

        self.assertTrue(browser.is_element_present_by_text('Token was returned successfully'), "Token return failed")

        browser.quit()

    def testAdminActions(self):
        self.adminActions(os.environ['TESTADMIN'], os.environ['TESTADMINPASS'])

    def testDomainAdminActions(self):
        self.adminActions(os.environ['TESTDOMAINADMIN'], os.environ['TESTDOMAINADMINPASS'])


if __name__ == '__main__':
    unittest.main()
