"""

    This tests the implemented ACLs
    Also these tests depend on each other and have to be run alphabetically

"""
import unittest
from os import environ

import requests

from token_helper import *
from univention.management.console.modules.twofactor_admin import disable_or_delete_all_tokens_for_user, rollout_qr_token, rollover_hardware_token
# from twofactor_admin import disable_or_delete_all_tokens_for_user, rollout_qr_token, rollover_hardware_token


environ['TESTADMIN'] ='Administrator'
environ['TESTADMINPASS']='xaver2root'
environ['TESTUSER1']='meier'
environ['TESTUSER1PASS']='12341234'
environ['TESTURL']='http://192.168.178.44'
environ['TESTPRIVACYIDEA_URL']='http://192.168.178.29'
environ['TESTPRIVACYIDEA_USER']='superuser'
environ['TESTPRIVACYIDEA_PASS']='xaver2root'
environ['TESTEMPLOYEE']='sek'
environ['TESTEMPLOYEEPASS']='seksek123'
environ['TESTEMPLOYEE2']='sek2'
environ['TESTEMPLOYEE2PASS']='sek2sek2123'
environ['TESTUSER2'] = 'schmidt'
environ['TESTUSER2PASS']='12341234'
environ['TESTDOMAINADMIN']='dadmin'
environ['TESTDOMAINADMINPASS'] ='dadm23rfjiofgd'
environ['TESTSTUDENT1']='teststudent'
environ['TESTSTUDENT1PASS']='dgshsfnnzdtndt'
environ['TESTEMPLOYEE3'] = 'sek3'
environ['TESTEMPLOYEE3PASS'] = 'dfgfghwsrt'
environ['TESTSTUDENT2'] ='teststudent2'
environ['TESTSTUDENT2PASS'] = 'dgssdfft'
environ['TESTUSER3'] = 'mueller'
environ['TESTUSER3PASS'] ='12341234'


baseurl = "%s/univention/" % environ['TESTURL']
#baseurl = "%s/" % environ['TESTURL']
twofactor_baseurl = baseurl + "command/twofactor-admin/"
#proxies = {"http": "http://localhost:8079", "https": "https://localhost:8079"}
#proxies = {"http": "http://192.168.178.25:8081", "https": "https://192.168.178.25:8081"}
proxies = {}


def search(uid, userlist):
    return [element for element in userlist if element['uid'] == uid]


class TestACLSs(unittest.TestCase):
    cookies = None
    headers = None
    linuser = None

    privacyidea_auth_token = None

    @classmethod
    def login(self, user, password):
        self.linuser = user
        res = requests.post(baseurl + "auth", json={"options": {"username": user, "password": password}},
                            cookies={"UMCLang": "en-US"}, proxies=proxies, verify=False)
        assert 200 == res.status_code
        assert res.cookies is not None
        self.cookies = res.cookies
        # pin language since some error messages are translated on the server
        requests.utils.add_dict_to_cookiejar(self.cookies, {"UMCLang": "en-US"})
        self.headers = {'X-Xsrf-Protection': res.cookies.get("UMCSessionId")}

    def logout(self):
        res = requests.get(baseurl + "/logout", cookies=self.cookies, headers=self.headers, proxies=proxies, verify=False)
        self.assertEqual(200, res.status_code)


    # get auth cookie
    @classmethod
    def setUpClass(self):
        self.privacyidea_auth_token = auth()
        delete_all_tokens_for_user(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token, environ['TESTUSER1'])
        delete_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token)
        create_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token)

    def assert_statuscode(self, school, user, statuscode):
        # Create a clean slate for the user to check
        print('School: ', school)
        print('User: ', user)
        print('Statuscode: ', statuscode)
        delete_all_tokens_for_user(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token, user)
        delete_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token)
        create_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token)

        if school:
            url = twofactor_baseurl + "set_current_school"
            res = requests.post(url, json={"options": {"school": school}}, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
            print('Url: ' + url)
            print('Response' + str(res))
            self.assertEqual(statuscode, res.status_code)

        # create qr token
        params = {'options': {'uid': ("%s" % user)}}
        res = requests.post(twofactor_baseurl + "rollover_qr_token", json=params, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        self.assertEqual(statuscode, res.status_code)

        # assign hardware token
        # params = {'options': {'uid': ("%s" % user), 'serial': serial, 'current_otp_value': get_current_otp()}}
        # res = requests.post(twofactor_baseurl + "assign_hardware_token", json=params, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        # self.assertEqual(statuscode, res.status_code)

    def assert_can_not_access(self, school, user):
        print()
        print('Logged in user: ', self.linuser,' assert_can_not_access: ', school, user)
        return self.assert_statuscode(school, user, 403)

    def assert_can_access(self, school, user):
        print()
        print('Logged in user: ', self.linuser, ' assert_can_access: ', school, user)
        return self.assert_statuscode(school, user, 200)

    def assert_can_not_list_users(self, school):
        # try to set school, outcome doesn't matter here, we're just interested in the user list
        print()
        print('Logged in user: ', self.linuser,'assert_can_not_list_users: ', school)
        if school:
            url = twofactor_baseurl + "set_current_school"
            res = requests.post(url, json={"options": {"school": school}}, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)

        # access user list
        url = twofactor_baseurl + "query"
        res = requests.post(url, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        self.assertEqual(403, res.status_code)

    def is_allmighty(self, username, password):
        self.logout()
        # This user is employee and school admin of DEMOSCHOOL
        self.login(username, password)
        print()
        print('Logged in user: ', self.linuser, ' is_allmighty: '+ username)


        # can access teachers of DEMOSCHOOL
        self.assert_can_access(None, environ['TESTUSER2'])

        # can access staff of DEMOSCHOOL
        self.assert_can_access(None, environ['TESTEMPLOYEE'])

        # can access teachers of ts
        self.assert_can_access(None, environ['TESTUSER1'])

        # can access staff of ts
        self.assert_can_access(None, environ['TESTEMPLOYEE2'])

        # can see all the users
        url = twofactor_baseurl + "query"
        res = requests.post(url, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        self.assertEqual(200, res.status_code)
        users = res.json()['result']

        self.assertIsNotNone(users)
        self.assertGreater(len(users), 9)

        # can query users by string
        res = requests.post(url, json={"options":{"pattern":"lehr"}}, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        users = res.json()['result']
        #self.assertEqual(3, len(users))
        # gets users from different schools
        self.assertTrue(any(environ['TESTEMPLOYEE'] in user['uid'] for user in users))
        self.assertTrue(any(environ['TESTEMPLOYEE2'] in user['uid'] for user in users))
        self.assertTrue(any(environ['TESTEMPLOYEE3'] in user['uid'] for user in users))

    def test_admin_is_almighty(self):
        print()
        print('-- test_admin_is_almighty --')
        self.is_allmighty(environ['TESTADMIN'], environ['TESTADMINPASS'])

    def test_domain_admin_is_allmighty(self):
        print()
        print('-- test_domain_admin_is_allmighty --')
        self.is_allmighty(environ['TESTDOMAINADMIN'], environ['TESTDOMAINADMINPASS'])

    def test_school_admin(self):
        self.logout()
        # This user is employee and school admin of DEMOSCHOOL
        self.login(environ['TESTEMPLOYEE'], environ['TESTEMPLOYEEPASS'])
        print()
        print('-- test_school_admin --')
        print('Logged in user: ', self.linuser, ' test_school_admin')

        # can access teachers of own school
        self.assert_can_access("DEMOSCHOOL", environ['TESTUSER2'])

        # can access staff of own school
        self.assert_can_access("DEMOSCHOOL", environ['TESTEMPLOYEE2'])

        # can access students of own school
        self.assert_can_access("DEMOSCHOOL", environ['TESTSTUDENT2'])

        # can't see teachers of other school
        url = twofactor_baseurl + "query"
        res = requests.post(url, cookies=self.cookies, headers=self.headers, verify=False, proxies=proxies)
        users = res.json()['result']
        self.assertEqual(200, res.status_code)
        self.assertNotIn(environ['TESTUSER1'], users)

        # can't access teachers of other school
        self.assert_statuscode("ts", environ['TESTUSER1'], 400)
        # can't access admin Account Tokens
        self.assert_statuscode(None, 'Administrator', 400)
        # can't access Domain Admin Account Tokens
        self.assert_statuscode(None, environ['TESTDOMAINADMIN'], 400)
        # can't access students of other school
        self.assert_statuscode("ts", environ['TESTSTUDENT1'], 400)
        # can't access staff of other school
        self.assert_statuscode("ts", environ['TESTEMPLOYEE3'], 400)


    def test_teacher(self):

        self.logout()
        self.login(environ['TESTUSER1'], environ['TESTUSER1PASS'])
        print()
        print('-- test_teacher --')
        print('Logged in user: ',self.linuser,' test_teacher')
        # can't access teachers of own school
        self.assert_can_not_access("ts", environ['TESTUSER1'])
        # can't access teachers of other school
        self.assert_can_not_access("DEMOSCHOOL", environ['TESTUSER2'])
        # can't access admin Account Tokens
        self.assert_can_not_access(None, 'Administrator')
        # can't access Domain Admin Account Tokens
        self.assert_can_not_access(None, environ['TESTDOMAINADMIN'])
        # can't access students of own school
        self.assert_can_not_access("ts", environ['TESTSTUDENT1'])
        # can't access staff of own school
        self.assert_can_not_access("ts", environ['TESTEMPLOYEE2'])
        # can't access staff of other school
        self.assert_can_not_access("DEMOSCHOOL", environ['TESTUSER2'])

    def test_student(self):

        self.logout()
        self.login(environ['TESTSTUDENT1'], environ['TESTSTUDENT1PASS'])
        print()
        print('-- test_student --')
        print('Logged in user: ',self.linuser,' test_student')
        # can't access teachers of own school
        self.assert_can_not_access("ts", environ['TESTUSER1'])
        # can't access teachers of other school
        self.assert_can_not_access("DEMOSCHOOL", environ['TESTUSER2'])
        # can't access admin Account Tokens
        self.assert_can_not_access(None, 'Administrator')
        # can't access Domain Admin Account Tokens
        self.assert_can_not_access(None, environ['TESTDOMAINADMIN'])
        # can't access students of own school
        self.assert_can_not_access("ts", environ['TESTSTUDENT1'])
        # can't access staff of own school
        self.assert_can_not_access("ts", environ['TESTEMPLOYEE2'])
        # can't access staff of other school
        self.assert_can_not_access("DEMOSCHOOL", environ['TESTEMPLOYEE'])

    def test_staff(self):

        self.logout()
        self.login(environ['TESTEMPLOYEE3'], environ['TESTEMPLOYEE3PASS'])
        print()
        print(' -- test_staff --')
        print('Logged in User: ',self.linuser, ' test_staff')
        # can't access teachers of own school
        self.assert_can_not_access("ts", environ['TESTUSER1'])

        # can't access teachers of other school
        self.assert_can_not_access("DEMOSCHOOL", environ['TESTUSER2'])

        # can't access admin Account Tokens
        self.assert_can_not_access(None, 'Administrator')
        # can't access Domain Admin Account Tokens
        self.assert_can_not_access(None, environ['TESTDOMAINADMIN'])
        # can't access students of own school
        self.assert_can_not_access("ts", environ['TESTSTUDENT1'])
        # can't access staff of own school
        self.assert_can_not_access("ts", environ['TESTEMPLOYEE2'])
        # can't access staff of other school
        self.assert_can_not_access("DEMOSCHOOL", environ['TESTEMPLOYEE'])









        # set a school
        # url = twofactor_baseurl + "set_current_school"
        # res = requests.post(url, json={"options": {"school": "ts"}}, cookies=self.cookies, headers=self.headers)
        # self.assertEqual(200, res.status_code)


if __name__ == '__main__':
    unittest.main(verbosity=2)
