"""

    This tests the usc jsonrpc implementation, doing the same stuff dojo will do eventually.
    The reasoning is faster development / divide an conquer: First get the backend right, then worry about the frontend

    needs TESTADMIN, TESTADMINPASS,TESTUSER1, TESTUSER1PASS and TESTURL Environment Variables
    # TESTEMPLOYEE=sek;TESTEMPLOYEEPASS=sek
    TESTEMPLOYEE2=sek2;TESTEMPLOYEE2PASS=sek2

    TESTEMPLOYEE2
       - needs to be admin of:



    Also these tests depend on each other and have to be run alphabetically

"""
import base64
import unittest
from os import environ

import requests

import token_helper
# from univention.management.console.modules.twofactor_admin.twofactor_admin import get_active_tokens_for_user
from .twofactor_admin import get_active_tokens_for_user
# from envs_vars import *

baseurl = "%s/univention/" % environ['TESTURL']
twofactor_baseurl = baseurl + "/command/twofactor-admin/"
# proxies = {"http": "http://localhost:8079", "https": "https://localhost:8079"}
proxies = {}


def search(uid, userlist):
    return [element for element in userlist if element['uid'] == uid]


class TestUmcEndpoints(unittest.TestCase):
    cookies = None
    headers = None

    privacyidea_auth_token = None

    @classmethod
    def login(self, user, password):
        res = requests.post(baseurl + "auth", json={"options": {"username": user, "password": password}},
                            cookies={"UMCLang": "en-US"}, proxies=proxies)
        assert 200 == res.status_code
        assert res.cookies is not None
        self.cookies = res.cookies
        # pin language since some error messages are translated on the server
        requests.utils.add_dict_to_cookiejar(self.cookies, {"UMCLang": "en-US"})
        self.headers = {'X-Xsrf-Protection': res.cookies.get("UMCSessionId")}

    def logout(self):
        res = requests.get(baseurl + "/logout", cookies=self.cookies, headers=self.headers, proxies=proxies)
        self.assertEqual(200, res.status_code)

    # get auth cookie
    @classmethod
    def setUpClass(self):
        self.login(environ['TESTEMPLOYEE2'], environ['TESTEMPLOYEE2PASS'])
        self.privacyidea_auth_token = token_helper.auth()
        token_helper.delete_all_tokens_for_user(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token, environ['TESTUSER1'])
        token_helper.delete_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token)
        token_helper.create_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token)

    def test_get_teachers(self):
        # set a school
        url = twofactor_baseurl + "set_current_school"
        res = requests.post(url, json={"options": {"school": "ts"}}, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)

        # query teachers
        url = twofactor_baseurl + "query"
        res = requests.post(url, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)
        self.assertIsNotNone(res.json()['result'])

    def test_rollover_qr_token(self):
        params = {'options': {'uid': ("%s" % environ['TESTUSER1'])}}
        res = requests.post(twofactor_baseurl + "rollover_qr_token", json=params, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)

        img = res.json()['result']['image']
        self.assertIsNotNone(img)

        imgdata = base64.b64decode(img.replace("data:image/png;base64,", ""))

        filename = 'qrcode.jpg'
        with open(filename, 'wb') as f:
            f.write(imgdata)

        # Test lost qr token (just gets deleted)
        res = requests.post(twofactor_baseurl + "lost_token", json=params, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)
        self.assertEqual("success", res.json()['result'])

        params = {'user': ("%s" % environ['TESTUSER1'])}
        r = requests.get(environ['TESTPRIVACYIDEA_URL'] + "/token/", params=params, headers={'Authorization': self.privacyidea_auth_token})
        self.assertEqual(200, res.status_code)
        res = r.json()

        tokens = get_active_tokens_for_user(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token, environ['TESTUSER1'])

        self.assertEqual(0, len(tokens), "There should be exactly zero active tokens for the user")

    def test_get_schools(self):
        res = requests.post(twofactor_baseurl + "/schools/query", data={"filter": ""}, cookies=self.cookies, headers=self.headers)
        print(res.json())
        self.assertEqual(200, res.status_code)

    def test_assign_hardware_token(self):
        self.login(environ['TESTEMPLOYEE2'], environ['TESTEMPLOYEE2PASS'])
        url = twofactor_baseurl + "set_current_school"
        res = requests.post(url, json={"options": {"school": "ts"}}, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)

        # Assign token to testuser 1
        params = {'options': {'uid': ("%s" % environ['TESTUSER1']), 'serial': token_helper.serial, 'current_otp_value': token_helper.get_current_otp()}}
        res = requests.post(twofactor_baseurl + "assign_hardware_token", json=params, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)
        self.assertEqual("success", res.json()['result'])

        # Try to assign qr token (should generate error since the active hardware token has not been returned / reported missing
        params = {'options': {'uid': ("%s" % environ['TESTUSER1'])}}
        res = requests.post(twofactor_baseurl + "rollover_qr_token", json=params, cookies=self.cookies, headers=self.headers)
        self.assertEqual(400, res.status_code)

    def test_assign_unknown_hardware_token(self):
        params = {'options': {'uid': ("%s" % environ['TESTUSER1']), 'serial': '123455', 'current_otp_value': '123456'}}
        res = requests.post(twofactor_baseurl + "assign_hardware_token", json=params, cookies=self.cookies, headers=self.headers, proxies=proxies)
        self.assertEqual(400, res.status_code)
        # self.assertEqual("Token not found", res.json()['message'], )

    def test_lost_and_found_hardware_token(self):
        params = {'options': {'uid': ("%s" % environ['TESTUSER1'])}}
        res = requests.post(twofactor_baseurl + "lost_token", json=params, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)
        self.assertEqual("success", res.json()['result'])

        params = {'user': ("%s" % environ['TESTUSER1'])}
        r = requests.get(environ['TESTPRIVACYIDEA_URL'] + "/token/", params=params, headers={'Authorization': self.privacyidea_auth_token})
        self.assertEqual(200, res.status_code)
        res = r.json()

        # make sure token is disabled and still assigned
        self.assertEqual(1, res['result']['value']['count'], "There should be exactly one disabled token for the user")
        self.assertEqual(res['result']['value']['tokens'][0]['active'], False)

        # return the token
        params = {'options': {'uid': ("%s" % environ['TESTUSER1']), 'serial': token_helper.serial, 'current_otp_value': token_helper.get_current_otp()}}
        res = requests.post(twofactor_baseurl + "return_hardware_token", json=params, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)
        self.assertEqual("success", res.json()['result'])

        # assert token is active and not assigned
        params = {'serial': ("%s" % token_helper.serial)}
        r = requests.get(environ['TESTPRIVACYIDEA_URL'] + "/token/", params=params, headers={'Authorization': self.privacyidea_auth_token})
        self.assertEqual(200, res.status_code)
        res = r.json()['result']
        self.assertEqual(res['value']['count'], 1)
        self.assertEqual(res['value']['tokens'][0]['active'], True)
        self.assertEqual(res['value']['tokens'][0]['user_id'], "")

    def test_assign_hardware_token_to_unknown_user(self):
        params = {'options': {'uid': ("%s" % "THISSHOULDREALLYNOTBEAUSERNAMEIFITISTHENWEGOTAFALSEPOSITIVE"), 'serial': token_helper.serial,
                              'current_otp_value': token_helper.get_current_otp()}}
        res = requests.post(twofactor_baseurl + "assign_hardware_token", json=params, cookies=self.cookies, headers=self.headers, proxies=proxies)
        self.assertEqual(400, res.status_code)

    def test_token_auth_against_user_from_other_school(self):
        self.logout()
        # this user is employee of school DEMOSCHOOL
        self.login(environ['TESTEMPLOYEE'], environ['TESTEMPLOYEEPASS'])

        # try to set unauth school
        url = twofactor_baseurl + "set_current_school"
        res = requests.post(url, json={"options": {"school": "ts"}}, cookies=self.cookies, headers=self.headers)
        # 400 Bad request
        # 403 Forbidden
        self.assertEqual(403, res.status_code)

        self.logout()
        self.login(environ['TESTEMPLOYEE'], environ['TESTEMPLOYEEPASS'])


        # TESTEMPLOYEE has no permission to set a school
        # set a school
        # url = twofactor_baseurl + "set_current_school"
        # res = requests.post(url, json={"options": {"school": "DEMOSCHOOL"}}, cookies=self.cookies, headers=self.headers)
        # self.assertEqual(200, res.status_code)

        # try to assign qr token
        params = {'options': {'uid': ("%s" % environ['TESTUSER1'])}}
        res = requests.post(twofactor_baseurl + "rollover_qr_token", json=params, cookies=self.cookies, headers=self.headers)
        # 400 Bad request
        # 403 Forbidden
        self.assertEqual(403, res.status_code)

        # try to assign hardware token
        params = {'options': {'uid': ("%s" % environ['TESTUSER1']), 'serial': token_helper.serial, 'current_otp_value': token_helper.get_current_otp()}}
        res = requests.post(twofactor_baseurl + "assign_hardware_token", json=params, cookies=self.cookies, headers=self.headers)
        # 400 Bad request
        # 403 Forbidden
        self.assertEqual(403, res.status_code)

        token_helper.assign_test_token(environ['TESTPRIVACYIDEA_URL'], self.privacyidea_auth_token, environ['TESTUSER1'])
        # try to repost lost token
        params = {'options': {'uid': ("%s" % environ['TESTUSER1'])}}
        res = requests.post(twofactor_baseurl + "lost_token", json=params, cookies=self.cookies, headers=self.headers)
        # 400 Bad request
        # 403 Forbidden
        self.assertEqual(403, res.status_code)

    def test_admin_is_no_school_admin(self):
        self.login(environ['TESTEMPLOYEE'], environ['TESTEMPLOYEEPASS'])
        # set a school for which Administrator is not a school admin
        url = twofactor_baseurl + "set_current_school"
        res = requests.post(url, json={"options": {"school": "ts"}}, cookies=self.cookies, headers=self.headers)
        #TESTEMPLOYEE has not permission to 2fa Frontend and should return a status code 403
        #see https://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html
        # 400 Bad request
        # 403 Forbidden
        #self.assertEqual(400, res.status_code)
        self.assertEqual(403, res.status_code)

        # query teachers without selection a school
        url = twofactor_baseurl + "query"
        res = requests.post(url, cookies=self.cookies, headers=self.headers)
        #TESTEMPLOYEE has not permission to 2fa Frontend and should return a status code 403
        #self.assertEqual(400, res.status_code)
        self.assertEqual(403, res.status_code)

    # def test_staff_sees_only_same_school_staff(self):
    #     self.login(environ['TESTEMPLOYEE'], environ['TESTEMPLOYEEPASS'])
    #     # set a school
    #     url = twofactor_baseurl + "set_current_school"
    #     res = requests.post(url, json={"options": {"school": "DEMOSCHOOL"}}, cookies=self.cookies, headers=self.headers)
    #     self.assertEqual(200, res.status_code)
    #     # query teachers
    #     url = twofactor_baseurl + "query"
    #     res = requests.post(url, cookies=self.cookies, headers=self.headers)
    #     self.assertEqual(200, res.status_code)
    #     self.assertIsNotNone(res.json()['result'])
    #     users = res.json()['result']
    #     self.assertFalse(search(environ['TESTUSER1'], users))

    # def test_2fa_admin_role(self):
    #     # Members of the group can use 2FA for their own school, others don't
    #     # TESTEMPLOYEE2 has admin rights
    #     self.login(environ['TESTEMPLOYEE2'], environ['TESTEMPLOYEE2PASS'])
    #     url = twofactor_baseurl + "set_current_school"
    #     res = requests.post(url, json={"options": {"school": "ts"}}, cookies=self.cookies, headers=self.headers)
    #     self.assertEqual(200, res.status_code)
    #     url = twofactor_baseurl + "query"
    #     res = requests.post(url, cookies=self.cookies, headers=self.headers)
    #     self.assertEqual(200, res.status_code)
    #     self.assertIsNotNone(res.json()['result'])
    #     users = res.json()['result']
    #     self.assertTrue(search(environ['TESTUSER1'], users))
    #     #in ucs-5  TESTEMPLOYEE2 is an mfa-admin and ist not in school ts consequently should not be tested
    #     #self.assertTrue(search(environ['TESTEMPLOYEE2'], users))

    #     self.login(environ['TESTEMPLOYEE'], environ['TESTEMPLOYEEPASS'])
    #     url = twofactor_baseurl + "set_current_school"
    #     res = requests.post(url, json={"options": {"school": "ts"}}, cookies=self.cookies, headers=self.headers)
    #     #TESTEMPLOYEE has not permission to 2fa Frontend and should return a status code 403
    #     #self.assertEqual(400, res.status_code)
    #     self.assertEqual(403, res.status_code)

    # def test_staff_and_teacher_list(self):
    #     # two roles must only be one entry in the list
    #     # fetch sek3
    #     self.login(environ['TESTEMPLOYEE2'], environ['TESTEMPLOYEE2PASS'])
    #     # set a school
    #     url = twofactor_baseurl + "set_current_school"
    #     res = requests.post(url, json={"options": {"school": "DEMOSCHOOL"}}, cookies=self.cookies, headers=self.headers)
    #     self.assertEqual(200, res.status_code)

    #     # query teachers
    #     url = twofactor_baseurl + "query"
    #     res = requests.post(url, cookies=self.cookies, headers=self.headers)
    #     self.assertEqual(200, res.status_code)
    #     self.assertIsNotNone(res.json()['result'])
    #     users = res.json()['result']
    #     self.assertEqual(1, len(search("sek", users)))

    def test_qr_token_unknown_user(self):
        params = {'options': {'uid': "%s" % "THISUSERNAMEHOPELFULLYDOESNOTEXIST"}}
        res = requests.post(twofactor_baseurl + "rollover_qr_token", json=params, cookies=self.cookies, headers=self.headers)
        self.assertEqual(400, res.status_code)


if __name__ == '__main__':
    unittest.main()
