# Creates a fake Hardware Token for testing purposes and calculates current OTP Values for this token
# The oath dependency is only for testing...
# This file does not (should not?) be deployed in production
# TODO: remove from dpkg

import binascii
import time
from os import environ

import requests
from hotpie import HOTP

# from univention.management.console.modules.twofactor_admin.twofactor_admin import get_auth_token
from twofactor_admin import get_auth_token


try:
    import univention.config_registry as ucr
    config = ucr.ConfigRegistry()
    config.load()
    realm = config.get('privacyidea/saml/realm')
except:
    realm = 'realdataport'

if realm is None:
    print("Please add a privacyidea/saml/realm Value or define the 'realm' Variable")
    exit(1)

otp_key = '3132333435363738393031323334353637383930'
serial = "123456789123456"
timestep = 1


def auth():
    return get_auth_token(environ['TESTPRIVACYIDEA_URL'], environ['TESTPRIVACYIDEA_USER'], environ['TESTPRIVACYIDEA_PASS'])


def create_test_token(url, auth_token):
    # delete test token
    r = requests.delete("%s/token/%s" % (url, serial), headers={'Authorization': auth_token})
    assert r.status_code == 200 or 404

    # create test token
    r = requests.post(url + "/token/init", headers={'Authorization': auth_token},
                      json={'serial': serial, "otpkey": otp_key, "type": "totp"})

    r = requests.post(url + "/token/realm/" + serial, headers={'Authorization': auth_token},
                      json={"realms": [realm]})

    assert 200 == r.status_code

    # make it a hardware token
    r = requests.post(url + "/token/info/%s/tokenkind" % serial, headers={'Authorization': auth_token},
                      json={'value': 'hardware'})

    assert 200 == r.status_code

    r = requests.post(url + "/token/info/%s/timeStep" % serial, headers={'Authorization': auth_token},
                      json={'value': timestep})

    assert 200 == r.status_code


def delete_test_token(url, auth_token):
    # deleting non existent tokens is an error
    r = requests.get(url + "/token/", params={"serial": serial}, headers={'Authorization': auth_token})
    assert 200 == r.status_code
    res = r.json()
    if res['result']['value']['count'] > 0:
        r = requests.delete(url + "/token/%s" % serial, headers={'Authorization': auth_token})
        assert 200 == r.status_code or 404 == r.status_code


def disable_test_token(url, auth_token):
    r = requests.post(url + "/token/disable", json={'serial': serial}, headers={'Authorization': auth_token})
    assert 200 == r.status_code


def enable_test_token(url, auth_token):
    r = requests.post(url + "/token/enable", json={'serial': serial}, headers={'Authorization': auth_token})
    assert 200 == r.status_code


def assign_test_token(url, auth_token, user):
    r = requests.post(url + "/token/assign", headers={'Authorization': auth_token},
                      json={'serial': str(serial), 'user': user})
    assert 200 == r.status_code


def unassign_test_token(url, auth_token, user):
    r = requests.post(url + "/token/unassign", headers={'Authorization': auth_token},
                      json={'serial': str(serial), 'user': user})
    assert 200 == r.status_code


# Deletes all Tokens for a user, only for testing purposes (hence this is not in twofactor_admin.py)
def delete_all_tokens_for_user(url, auth_token, user):
    r = requests.get(url + "/token", headers={'Authorization': auth_token}, json={'user': user})
    result = r.json()['result']

    if r.status_code == 200:
        for tfa_token in result['value']['tokens']:
            r = requests.delete(url + "/token/%s" % tfa_token['serial'], headers={'Authorization': auth_token})
            if r.status_code != 200:
                raise ValueError("could not delete token")


def get_current_otp():
    k = binascii.unhexlify(otp_key.lower())

    otp = HOTP(k, int(int(time.time() / timestep)) + 1)
    print("otp %s" % otp)

    return otp
